"""
Handles initialising pygame for gp2x hardware
Provides Constants for gp2x hardware
Provides a basic menu system.
"""
import pygame, os, sys, getopt
from pygame.locals import *
clock = pygame.time.Clock()
FRAME_RATE = 12

MAX_WIDTH = 16384  #width of surface

# GP2X joystick button mappings
BUTTON_UP      = (0)
BUTTON_DOWN      = (4)
BUTTON_LEFT      = (2)
BUTTON_RIGHT     = (6)
BUTTON_UPLEFT    = (1)
BUTTON_UPRIGHT     = (7)
BUTTON_DOWNLEFT    = (3)
BUTTON_DOWNRIGHT   = (5)
BUTTON_CLICK     = (18)
BUTTON_A       = (12)
BUTTON_B       = (13)
BUTTON_X       = (15)
BUTTON_Y       = (14)
BUTTON_L       = (10)
BUTTON_R       = (11)
BUTTON_START     = (8) 
BUTTON_SELECT    = (9)
BUTTON_VOLUP     = (16)  #hmm, pygame might not map to these
BUTTON_VOLDOWN     = (17)

#all hardware gets mapped to these.
KEY_A 		= 1
KEY_B 		= 2
KEY_SELECT	= 4
KEY_START 	= 8
KEY_RIGHT 	= 16
KEY_LEFT 	= 32
KEY_UP 		= 64
KEY_DOWN 	= 128
KEY_R		= 256
KEY_L 		= 512
KEY_X = 1024
KEY_Y = 2048
KEY_UPLEFT    = 4096
KEY_UPRIGHT     = 8192
KEY_DOWNLEFT    = 16384
KEY_DOWNRIGHT   = 32768
KEY_VOLUP = 65536
KEY_VOLDOWN = 131072

SCREEN_DEPTH = 8

size = width, height = 320, 240
black = 0, 0, 0

portable = False

"""
Three functions for multiline text boxes
#from: http://trac.codereactor.net/pygext/browser/trunk/pygext/text/__init__.py?rev=1&format=txt
pygext.text - Graphical text utilities
author: Sami Hangaslammi (shang@jyu.fi)
"""
def textbox(font, text, maxwidth, position=(0,0),colour=(255,255,255), antialias=1, surface = None):
    """textbox(font, text, maxwidth, colour=(255,255,255), antialias=1) -> Surface

    Renders a list of strings using the given font so that each line is no
    wider than the specified maximum width (exception: a word does
    fit a single line). This is essentially the same as calling
    pygext.text.split and pygext.text.multiline manually.
    """
    finallines = []
#   	lines = text.split("\n")
    for line in text:
        finallines.extend(split(font, line, maxwidth))
    s = multiline(font, finallines, colour, antialias, position=position,surface=surface)
    return s


def split(font, text, maxwidth):
    """split(font, text, maxwidth) -> lines

    Splits a string of text into multiple lines, so that
    when the text is rendered using the given font, no
    line exceeds the specified maximum width.
    """    	
    text = text.strip('\n')
    words = text.split(' ')
    words.reverse()
    line = []
    lines = []
    nextword = words.pop()
    while 1:
        linetext = ' '.join(line+[nextword])
        width,height = font.size(linetext)
        if line and width > maxwidth:
            lines.append(' '.join(line))
            line = []
        else:
            line.append(nextword)
            if not words:
                if line:
                    lines.append(' '.join(line))
                    width,height = font.size(linetext)
                break
            nextword = words.pop()
    return lines

def multiline(font, lines, colour=(255,255,255), antialias=1, surface = None,position=[0,0]):
    """multiline(font, lines) -> Surface

    Renders multiple lines of text. Lines can be string
    that contains linefeed characters or a list of strings.
    """
#    print "s",surface
    s = surface
    if isinstance(lines, str):
        lines = lines.split("\n")
    maxwidth = 0
    height = 0
    ## calculate surface size
    for line in lines:
        w,h = font.size(line)
        maxwidth = max(maxwidth, w)
        height += h
	if s == None:
	    s = pygame.Surface((maxwidth,height))
	    s.fill((0,0,0,0))
    y = position[1]
    for line in lines:
        if line:
            s2 = font.render(line, antialias, colour)
            s.blit(s2, (position[0],y))
            y += s2.get_size()[1]
        else:
            w,h = font.size(line)
            y += h
    return s





def loadImage(fname):
		"""
		Load an image and return it.
		"""
		image = None
 		if not os.path.isfile(fname):
			print fname+" not found."
			return None
		try:
			image = pygame.image.load(fname)
		except Exception, e:
			print "vida: Problem loading image",fname,repr(e), e, Exception,". vida will probably exit soon."
			return None

		if image.get_alpha() is None:
			image = image.convert()
			image.set_colorkey(image.get_at((0,0)), RLEACCEL)
		else:
			image = image.convert_alpha()
		return image

def loadImage_noAlpha(fname):
		"""
		Load an image and return it.
		"""
		image = None
 		if not os.path.isfile(fname):
			print fname+" not found."
			return None
		try:
			image = pygame.image.load(fname)
		except Exception, e:
			print "vida: Problem loading image",fname,repr(e), e, Exception,". vida will probably exit soon."
			return None

		image = image.convert()
		"""
		if image.get_alpha() is None:
			image = image.convert()
			image.set_colorkey(image.get_at((0,0)), RLEACCEL)
		else:
			image = image.convert_alpha()
		"""
		return image


class gp2xtra_general:
  def __init__(self):
	self.portable = False #PC or Portable (gp2x?)
	self.running = 1
	self.screen = None
	self.thisKeys = 0
	self.lastKeys = 0
	self.repeated = 15
	self.textColour = None
	self.allowPause = True
	self.paused = False
	self.background = None
	self.key_functions = {}  #key functions to check for
	self.keyNB_functions = {}  #key functions to check for (no bounce)



def initialise_pygame():
  pygame.init()
  num_joysticks = pygame.joystick.get_count()
  if num_joysticks > 0:
    print "Initialised joystick"
    stick = pygame.joystick.Joystick(0)
    stick.init() # now we will receive events for the joystick
  screen = pygame.display.set_mode(size, pygame.FULLSCREEN & pygame.HWSURFACE)
  return screen


def gp2xtra_init():
  screen = initialise_pygame()
  sx, sy = screen.get_rect().width, screen.get_rect().height
  return screen

def gp2xtra_input_test():
  portable = False
  try:
      opts, args = getopt.getopt(sys.argv[1:], "p", ["portable"])
  except getopt.error, msg:
      print msg
      print "for portables uses -p"
      sys.exit(2)
  for o, a in opts:
      if o in ("-p", "--portable"):
        portable = True
  if portable == True:
    sys.stderr.write("\n\nUsing portable keys\n\n")
    pygame.mouse.set_visible(0)
  else:
    sys.stderr.write("\n\nUsing PC keyboard\n\n")
  return portable

def gp2xtra_list(screen, items, backgroundImage = None):  #stub for our menu function
  return items[0]

"""
The keyboard stuff is also gp2x specific.
"""

def KeyPressed(game, key):
  """
  Test if "key" was was pressed
  """
#  print game.thisKeys, key
  if (game.thisKeys & key):
#    print "match!", key
    return 1
  return 0


#thisKeys = currently pressed down keys
#lastKeys = keys pressed down on previous poll
#r is the allowed bounce for a key.
def KeyPressedNoBounce(game, key, r):
  if (game.thisKeys & key): #// key is currently pressed
	if (game.lastKeys & key): #was pressed last time.
		game.repeated += 1
		if game.repeated>=r:  #only trigger key if it is longer than allowed bounce.
			game.repeated = 0
			return 1
		else: #key is repeated, so no new event.
		    return 0;  
	else:
#		game.repeated = 0  
		return 1
  else: #key not currently pressed
#    if ((game.lastKeys & key) or (not (game.lastKeys & key) and r < game.repeated)): #// and it wasn't pressed last time..
      game.repeated = 0;
  return 0;


def InitKeys(game):
  game.lastKeys = 0xFFFF;
  game.thisKeys = 0xFFFF;


def UpdateKeys(game):
  if game.portable == True:
   gp2xUpdateKeys(game)
  else:
   pcUpdateKeys(game)


def pcUpdateKeys(game):
  game.lastKeys = game.thisKeys;
  running = 1
  KEYS = game.thisKeys
  c_blue = 0, 0, 255
  while running:
#    print "keys",KEYS
    event = pygame.event.poll()
    if event.type == pygame.NOEVENT:
      running = 0
    elif event.type == KEYDOWN or event.type == KEYUP:
#     key = pygame.key.name(event.key)
      if event.key == K_ESCAPE:
        game.running = 0
      if event.key == K_x:
        KEYS = KEYS ^ KEY_X
      elif event.key == K_a:
        KEYS = KEYS ^ KEY_A
      elif event.key == K_b:
        KEYS = KEYS ^ KEY_B
      elif event.key == K_c:
        KEYS = KEYS ^ KEY_SELECT
        print "KEY SELECT"
      elif event.key == K_v:
        KEYS = KEYS ^ KEY_START
        #gba_pixel(game.screen, c_blue, (12,230) )
        print "KEY START"
      elif event.key == K_r:
        KEYS = KEYS ^ KEY_R
      elif event.key == K_l:
        KEYS = KEYS ^ KEY_L
      elif event.key == K_y:
        KEYS = KEYS ^ KEY_Y
      elif event.key == K_UP:
        KEYS = KEYS ^ KEY_UP
      elif event.key == K_DOWN:
        KEYS = KEYS ^ KEY_DOWN
      elif event.key == K_LEFT:
        KEYS = KEYS ^ KEY_LEFT
      elif event.key == K_RIGHT:
        KEYS = KEYS ^ KEY_RIGHT
      elif event.key == K_2:
        KEYS = KEYS ^ KEY_VOLUP
      elif event.key == K_1:
        KEYS = KEYS ^ KEY_VOLDOWN

  game.thisKeys = KEYS

def gp2xUpdateKeys(game):
  game.lastKeys = game.thisKeys;
  running = 1
  """
KEY_RIGHT 	= 16
KEY_LEFT 		= 32
KEY_UP 		= 64
KEY_DOWN 		= 128
  """
  KEYS = game.thisKeys
  c_blue = 0, 0, 255
#  print game.repeated
  while running:
    event = pygame.event.poll()
    if event.type == pygame.NOEVENT:
      running = 0
    elif event.type == pygame.JOYBUTTONDOWN or event.type == pygame.JOYBUTTONUP:
      if event.button == BUTTON_X:
        KEYS = KEYS ^ KEY_X
      elif event.button == BUTTON_A:
        KEYS = KEYS ^ KEY_A
      elif event.button == BUTTON_B:
        KEYS = KEYS ^ KEY_B
      elif event.button == BUTTON_SELECT:
        KEYS = KEYS ^ KEY_SELECT
      elif event.button == BUTTON_START:
#        gba_pixel(game.screen, c_blue, (12,230) )
        KEYS = KEYS ^ KEY_START
      elif event.button == BUTTON_R:
        KEYS = KEYS ^ KEY_R
      elif event.button == BUTTON_L:
        KEYS = KEYS ^ KEY_L
      elif event.button == BUTTON_Y:
        KEYS = KEYS ^ KEY_Y
      elif event.button == BUTTON_UP:
        KEYS = KEYS ^ KEY_UP
      elif event.button == BUTTON_DOWN:
        KEYS = KEYS ^ KEY_DOWN
      elif event.button == BUTTON_LEFT:
        KEYS = KEYS ^ KEY_LEFT
      elif event.button == BUTTON_RIGHT:
        KEYS = KEYS ^ KEY_RIGHT
      elif event.button == BUTTON_UPLEFT:
        KEYS = KEYS ^ KEY_UPLEFT
      elif event.button == BUTTON_UPRIGHT:
        KEYS = KEYS ^ KEY_UPRIGHT
      elif event.button == BUTTON_DOWNLEFT:
        KEYS = KEYS ^ KEY_DOWNLEFT
      elif event.button == BUTTON_DOWNRIGHT:
        KEYS = KEYS ^ KEY_DOWNRIGHT
      elif event.button == BUTTON_VOLUP:
        KEYS = KEYS ^ KEY_VOLUP
      elif event.button == BUTTON_VOLDOWN:
        KEYS = KEYS ^ KEY_VOLDOWN

  game.thisKeys = KEYS
